;;; File: circ-focus.lisp
;;; New systems, choosers, inquries, and gates for the circumstantial adjuncts
;;; These are from Susan Williams, May, 1995.
;;;
;;; If this file is loaded, the following functions should be called by the
;;;   loading program:
;;;   (reset-system-network)
;;;   (set-grammar-keyword-types)
;;; and the file finalfix.lisp should be loaded either before or after this
;;; file.

(in-package :PENMAN)


;;;----------------------------------------------------------------------------
;;; New systems, choosers, and inquiries for ACCOMPANIMENT
(system
 :name ACCOMPANIMENT-salience
 :inputs nonthematic-ACCOMPANIMENT
 :outputs ((0.50 default-accompaniment)
	   (0.10 initial-accompaniment
		 (insert initial)
		 (expand initial accompaniment))
	   (0.20 medial-ACCOMPANIMENT
		 (insert medial)
		 (expand medial accompaniment))
	   (0.10 initial-end-ACCOMPANIMENT
		 (insert initial-end)
		 (EXPAND initial-end ACCOMPANIMENT))
	   (0.10 end-accompaniment
		 (insert end)
		 (expand end accompaniment)))
 :chooser ACCOMPANIMENT-salience-CHOOSER
 :selector CHOICE-MASTER
 :old-feature NONTHEMATIC-ACCOMPANIMENT
 :region THEME
 :metafunction TEXTUAL)

(chooser
 :name ACCOMPANIMENT-salience-CHOOSER 
 :definition ((ASK (default-accompaniment-salience-q accompaniment onus)
		   (default-salience (choose default-accompaniment))
		   (nondefault-salience
		    (ask (accompaniment-salience-q accompaniment onus)
			 (initial (choose initial-accompaniment))
			 (medial (choose medial-accompaniment))
			 (initial-end (choose initial-end-accompaniment))
			 (end (choose end-accompaniment))))))
 :comments ""
 :editor "WW"
 :date "15-mar-95")

(askoperator 
 :name accompaniment-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english
   ("In which position should " CIRCUMSTANCE " appear relative to " SPEC " ?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (initial medial initial-end end)
 :activedefault medial
 :trivialdefault medial)

(askoperator 
 :name default-accompaniment-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english
 ("Should " CIRCUMSTANCE " appear in a default position relative to " SPEC "?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (default-salience nondefault-salience)
 :activedefault default-salience
 :trivialdefault default-salience)


(set-inquiry-mode-deimplemented 'accompaniment-salience-Q)
(set-inquiry-mode-deimplemented 'default-accompaniment-salience-Q)

;;;----------------------------------------------------------------------------
;;; New systems, choosers, and inquiries for MANNER

(system
 :name MANNER-salience
 :inputs nonthematic-MANNER
 :outputs ((0.50 default-manner)
	   (0.10 initial-manner
		 (insert initial)
		 (expand initial manner))
	   (0.20 medial-MANNER
		 (insert medial)
		 (expand medial manner))
	   (0.10 initial-end-MANNER
		 (insert initial-end)
		 (EXPAND initial-end MANNER))
	   (0.10 end-manner
		 (insert end)
		 (expand end manner)))
 :chooser MANNER-salience-CHOOSER
 :selector CHOICE-MASTER
 :old-feature NONTHEMATIC-MANNER
 :region THEME
 :metafunction TEXTUAL)

(chooser
 :name MANNER-salience-CHOOSER 
 :definition ((ASK (default-manner-salience-q manner onus)
		   (default-salience (choose default-manner))
		   (nondefault-salience
		    (ask (manner-salience-q manner onus)
			 (initial (choose initial-manner))
			 (medial (choose medial-manner))
			 (initial-end (choose initial-end-manner))
			 (end (choose end-manner))))))
 :comments ""
 :editor "WW"
 :date "15-mar-95")

(askoperator 
 :name manner-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english
   ("In which position should " CIRCUMSTANCE " appear relative to " SPEC " ?") 
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (initial medial initial-end end)
 :activedefault medial
 :trivialdefault medial)

(askoperator 
 :name default-manner-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
 ("Should " CIRCUMSTANCE " appear in a default position relative to " SPEC "?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (default-salience nondefault-salience)
 :activedefault default-salience
 :trivialdefault default-salience)


(set-inquiry-mode-deimplemented 'manner-salience-Q)
(set-inquiry-mode-deimplemented 'default-manner-salience-Q)

;;;---------------------------------------------------------------------------
;;; New systems, choosers, and inquiries for MATTER

(system
 :name MATTER-salience
 :inputs nonthematic-MATTER
 :outputs ((0.50 default-matter)
	   (0.10 initial-matter
		 (insert initial)
		 (expand initial matter))
	   (0.20 medial-MATTER
		 (insert medial)
		 (expand medial matter))
	   (0.10 initial-end-MATTER
		 (insert initial-end)
		 (EXPAND initial-end MATTER))
	   (0.10 end-matter
		 (insert end)
		 (expand end matter)))
 :chooser MATTER-salience-CHOOSER
 :selector CHOICE-MASTER
 :old-feature NONTHEMATIC-MATTER
 :region THEME
 :metafunction TEXTUAL)

(chooser
 :name MATTER-salience-CHOOSER 
 :definition ((ASK (default-matter-salience-q matter onus)
		   (default-salience (choose default-matter))
		   (nondefault-salience
		    (ask (matter-salience-q matter onus)
			 (initial (choose initial-matter))
			 (medial (choose medial-matter))
			 (initial-end (choose initial-end-matter))
			 (end (choose end-matter))))))
 :comments ""
 :editor "WW"
 :date "15-mar-95")

(askoperator 
 :name matter-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
   ("In which position should " CIRCUMSTANCE " appear relative to " SPEC " ?") 
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (initial medial initial-end end)
 :activedefault medial
 :trivialdefault medial)

(askoperator 
 :name default-matter-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
 ("Should " CIRCUMSTANCE " appear in a default position relative to " SPEC "?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (default-salience nondefault-salience)
 :activedefault default-salience
 :trivialdefault default-salience)


(set-inquiry-mode-deimplemented 'matter-salience-Q)
(set-inquiry-mode-deimplemented 'default-matter-salience-Q)

;;;----------------------------------------------------------------------------
;;; New systems, choosers, and inquiries for TEMPORAL-EXTENT

(system
 :name TEMPORAL-EXTENT-salience
 :inputs nonthematic-TEMPORAL-EXTENT
 :outputs ((0.50 default-temporal-extent)
	   (0.10 initial-temporal-extent
		 (insert initial)
		 (expand initial timeextent))
	   (0.20 medial-TEMPORAL-EXTENT
		 (insert medial)
		 (expand medial timeextent))
	   (0.10 initial-end-TEMPORAL-EXTENT
		 (insert initial-end)
		 (EXPAND initial-end TIMEEXTENT))
	   (0.10  end-temporal-extent
		 (insert end)
		 (expand end timeextent)))
 :chooser TEMPORAL-EXTENT-salience-CHOOSER
 :selector CHOICE-MASTER
 :old-feature NONTHEMATIC-TEMPORAL-EXTENT
 :region THEME
 :metafunction TEXTUAL)

(chooser
 :name TEMPORAL-EXTENT-salience-CHOOSER 
 :definition ((ASK (default-temporal-extent-salience-q timeextent onus)
		   (default-salience (choose default-temporal-extent))
		   (nondefault-salience
		    (ask (temporal-extent-salience-q timeextent onus)
			 (initial (choose initial-temporal-extent))
			 (medial (choose medial-temporal-extent))
			 (initial-end (choose initial-end-temporal-extent))
			 (end (choose end-temporal-extent))))))
 :comments ""
 :editor "WW"
 :date "15-mar-95")

(askoperator 
 :name temporal-extent-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
   ("In which position should " CIRCUMSTANCE " appear relative to " SPEC " ?") 
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (initial medial initial-end end)
 :activedefault medial
 :trivialdefault medial)

(askoperator 
 :name default-temporal-extent-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
 ("Should " CIRCUMSTANCE " appear in a default position relative to " SPEC "?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (default-salience nondefault-salience)
 :activedefault default-salience
 :trivialdefault default-salience)


(set-inquiry-mode-deimplemented 'temporal-extent-salience-Q)
(set-inquiry-mode-deimplemented 'default-temporal-extent-salience-Q)

;;;----------------------------------------------------------------------------
;;; New systems, choosers, and inquiries for SPATIAL-EXTENT

(system
 :name SPATIAL-EXTENT-salience
 :inputs nonthematic-SPATIAL-EXTENT
 :outputs ((0.50 default-spatial-extent)
	   (0.10 initial-spatial-extent
		 (insert initial)
		 (expand initial spaceextent))
	   (0.20 medial-SPATIAL-EXTENT
		 (insert medial)
		 (expand medial spaceextent))
	   (0.10 initial-end-SPATIAL-EXTENT
		 (insert initial-end)
		 (EXPAND initial-end SPACEEXTENT))
	   (0.10 end-spatial-extent
		 (insert end)
		 (expand end spaceextent)))
 :chooser SPATIAL-EXTENT-salience-CHOOSER
 :selector CHOICE-MASTER
 :old-feature NONTHEMATIC-SPATIAL-EXTENT
 :region THEME
 :metafunction TEXTUAL)

(chooser
 :name SPATIAL-EXTENT-salience-CHOOSER 
 :definition ((ASK (default-spatial-extent-salience-q spaceextent onus)
		   (default-salience (choose default-spatial-extent))
		   (nondefault-salience
		    (ask (spatial-extent-salience-q spaceextent onus)
			 (initial (choose initial-spatial-extent))
			 (medial (choose medial-spatial-extent))
			 (initial-end (choose initial-end-spatial-extent))
			 (end (choose end-spatial-extent))))))
 :comments ""
 :editor "WW"
 :date "15-mar-95")

(askoperator 
 :name spatial-extent-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
   ("In which position should " CIRCUMSTANCE " appear relative to " SPEC " ?") 
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (initial medial initial-end end)
 :activedefault medial
 :trivialdefault medial)

(askoperator 
 :name default-spatial-extent-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
 ("Should " CIRCUMSTANCE " appear in a default position relative to " SPEC "?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (default-salience nondefault-salience)
 :activedefault default-salience
 :trivialdefault default-salience)


(set-inquiry-mode-deimplemented 'spatial-extent-salience-Q)
(set-inquiry-mode-deimplemented 'default-spatial-extent-salience-Q)

;;;----------------------------------------------------------------------------
;;; New systems, choosers, and inquiries for ROLE

(system
 :name ROLE-salience
 :inputs nonthematic-ROLE
 :outputs ((0.50 default-role)
	   (0.10 initial-role
		 (insert initial)
		 (expand initial role))
	   (0.20 medial-ROLE
		 (insert medial)
		 (expand medial role))
	   (0.10 initial-end-ROLE
		 (insert initial-end)
		 (EXPAND initial-end ROLE))
	   (0.10 end-role
		 (insert end)
		 (expand end role)))
 :chooser ROLE-salience-CHOOSER
 :selector CHOICE-MASTER
 :old-feature NONTHEMATIC-ROLE
 :region THEME
 :metafunction TEXTUAL)

(chooser
 :name ROLE-salience-CHOOSER 
 :definition ((ASK (default-role-salience-q role onus)
		   (default-salience (choose default-role))
		   (nondefault-salience
		    (ask (role-salience-q role onus)
			 (initial (choose initial-role))
			 (medial (choose medial-role))
			 (initial-end (choose initial-end-role))
			 (end (choose end-role))))))
 :comments ""
 :editor "WW"
 :date "15-mar-95")

(askoperator 
 :name role-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
   ("In which position should " CIRCUMSTANCE " appear relative to " SPEC " ?") 
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (initial medial initial-end end)
 :activedefault medial
 :trivialdefault medial)

(askoperator 
 :name default-role-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
 ("Should " CIRCUMSTANCE " appear in a default position relative to " SPEC "?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (default-salience nondefault-salience)
 :activedefault default-salience
 :trivialdefault default-salience)


(set-inquiry-mode-deimplemented 'role-salience-Q)
(set-inquiry-mode-deimplemented 'default-role-salience-Q)

;;;----------------------------------------------------------------------------
;;; New systems, choosers, and inquiries for SPATIAL-LOCATION

(system
 :name SPATIAL-LOCATION-salience
 :inputs nonthematic-SPATIAL-LOCATION
 :outputs ((0.50 default-spatial-location)
	   (0.10 initial-spatial-location
		 (insert initial)
		 (expand initial spacelocative))
	   (0.20 medial-SPATIAL-LOCATION
		 (insert medial)
		 (expand medial spacelocative))
	   (0.10 initial-end-SPATIAL-LOCATION
		 (insert initial-end)
		 (EXPAND initial-end SPACELOCATIVE))
	   (0.10 end-spatial-location
		 (insert end)
		 (expand end spacelocative)))
 :chooser SPATIAL-LOCATION-salience-CHOOSER
 :selector CHOICE-MASTER
 :old-feature NONTHEMATIC-SPATIAL-LOCATION
 :region THEME
 :metafunction TEXTUAL)

(chooser
 :name SPATIAL-LOCATION-salience-CHOOSER 
 :definition ((ASK (default-spatial-location-salience-q spacelocative onus)
		   (default-salience (choose default-spatial-location))
		   (nondefault-salience
		    (ask (spatial-location-salience-q spacelocative onus)
			 (initial (choose initial-spatial-location))
			 (medial (choose medial-spatial-location))
			 (initial-end (choose initial-end-spatial-location))
			 (end (choose end-spatial-location))))))
 :comments ""
 :editor "WW"
 :date "15-mar-95")

(askoperator 
 :name spatial-location-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
   ("In which position should " CIRCUMSTANCE " appear relative to " SPEC " ?") 
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (initial medial initial-end end)
 :activedefault medial
 :trivialdefault medial)

(askoperator 
 :name default-spatial-location-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
 ("Should " CIRCUMSTANCE " appear in a default position relative to " SPEC "?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (default-salience nondefault-salience)
 :activedefault default-salience
 :trivialdefault default-salience)


(set-inquiry-mode-deimplemented 'spatial-location-salience-Q)
(set-inquiry-mode-deimplemented 'default-spatial-location-salience-Q)

;;;----------------------------------------------------------------------------
;;; New systems, choosers, and inquiries for TEMPORAL-LOCATION

(system
 :name TEMPORAL-LOCATION-salience
 :inputs nonthematic-TEMPORAL-LOCATION
 :outputs ((0.50 default-temporal-location)
	   (0.10 initial-temporal-location
		 (insert initial)
		 (expand initial timelocative))
	   (0.20 medial-TEMPORAL-LOCATION
		 (insert medial)
		 (expand medial timelocative))
	   (0.10 initial-end-TEMPORAL-LOCATION
		 (insert initial-end)
		 (EXPAND initial-end TIMELOCATIVE))
	   (0.10 end-temporal-location
		 (insert end)
		 (expand end timelocative)))
 :chooser TEMPORAL-LOCATION-salience-CHOOSER
 :selector CHOICE-MASTER
 :old-feature NONTHEMATIC-TEMPORAL-LOCATION
 :region THEME
 :metafunction TEXTUAL)

(chooser
 :name TEMPORAL-LOCATION-salience-CHOOSER 
 :definition ((ASK (default-temporal-location-salience-q timelocative onus)
		   (default-salience (choose default-temporal-location))
		   (nondefault-salience
		    (ask (temporal-location-salience-q timelocative onus)
			 (initial (choose initial-temporal-location))
			 (medial (choose medial-temporal-location))
			 (initial-end (choose initial-end-temporal-location))
			 (end (choose end-temporal-location))))))
 :comments ""
 :editor "WW"
 :date "15-mar-95")

(askoperator 
 :name temporal-location-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
   ("In which position should " CIRCUMSTANCE " appear relative to " SPEC " ?") 
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (initial medial initial-end end)
 :activedefault medial
 :trivialdefault medial)

(askoperator 
 :name default-temporal-location-salience-Q
 :domain TP
 :mode IMPLEMENTED
 :parameters (CIRCUMSTANCE SPEC)
 :english 
 ("Should " CIRCUMSTANCE " appear in a default position relative to " SPEC "?")
 :operatorcode trivialdefaultcode
 :parameterassociationtypes (CONCEPT CONCEPT)
 :answerset (default-salience nondefault-salience)
 :activedefault default-salience
 :trivialdefault default-salience)


(set-inquiry-mode-deimplemented 'temporal-location-salience-Q)
(set-inquiry-mode-deimplemented 'default-temporal-location-salience-Q)

;;;----------------------------------------------------------------------------
;;; These gates were changed so that NEGATOR would be expanded with MOOD.
;;; This does not seem to affect any thing else in the grammar and made the
;;; writing of gates for the medial position much easier.

;;; feature NEGATIVE was renamed to NEGATIVE-AUX in KPML 1.0 - Jon 21/07/97

(gate
   :name NEG-IMPERATIVE-FINITE
   :inputs (AND NEGATIVE IMPERATIVE)
   :outputs ((1.0 NEG-IMPERATIVE-FINITE 
		  (INSERT NEGATOR) 
		  (expand mood negator)
		  (CLASSIFY NEGATOR NEGATIVE-aux) 
		  (CLASSIFY NEGATOR ADVERB)
		  (OUTCLASSIFY FINITE NEGATIVE-aux)))
   :selector CHOICE-MASTER
   :old-feature NEG-INDICATIVE-FINITE
   :region MOOD
   :metafunction INTERPERSONAL)

(gate
   :name NEG-INDICATIVE-FINITE
   :inputs (AND NEGATIVE
                (OR INDICATIVE
                    INDIRECT-INDICATIVE
                    EXPANSION-CLAUSE))
   :outputs ((1.0 NEG-INDICATIVE-FINITE 
		  (INSERT NEGATOR)
		  (expand mood negator)
		  (CLASSIFY NEGATOR NEGATIVE-aux)
		  (CLASSIFY NEGATOR ADVERB)
		  (OUTCLASSIFY FINITE NEGATIVE-aux)))
   :selector CHOICE-MASTER
   :old-feature NEG-INDICATIVE-FINITE
   :region MOOD
   :metafunction INTERPERSONAL)

;;;----------------------------------------------------------------------------
;;; And now the new gates that I wrote:

(gate
 :name medial-ORDER-after-mood
 :inputs (and (or future secondary modal
		  finite-negator-subject
		  finite-subject-negator
		  finite-negator
		  finite-subject
		  passive-process)
	      (or medial-accompaniment
		  medial-manner
		  medial-matter
		  medial-temporal-extent
		  medial-spatial-extent
		  medial-role
		  medial-spatial-location
		  medial-temporal-location))
 :outputs ((1.0 medial-ORDER-after-mood
		(order-anchor-first mood medial)))
 :selector CHOICE-MASTER
 :old-feature THEME-ORDER
 :region THEME
 :metafunction TEXTUAL)

(gate
 :name medial-ORDER-after-subject
 :inputs (and (or medial-accompaniment
		  medial-manner
		  medial-matter
		  medial-temporal-extent
		  medial-spatial-extent
		  medial-role
		  medial-spatial-location
		  medial-temporal-location)
	      not-auxed
	      positive
	      declarative)
 :outputs ((1.0 medial-ORDER-after-subject
		(order-anchor-first subject medial)))
 :selector CHOICE-MASTER
 :old-feature THEME-ORDER
 :region THEME
 :metafunction TEXTUAL)


(gate
 :name initial-end-ORDER
 :inputs (or initial-end-accompaniment
	     initial-end-manner
	     initial-end-matter
	     initial-end-temporal-extent
	     initial-end-spatial-extent
	     initial-end-role
	     iniital-end-spatial-location
	     initial-end-temporal-location)
 :outputs ((1.0 initial-end-ORDER
		(ORDER-anchor-first lexverb initial-end)))
 :selector CHOICE-MASTER
 :old-feature THEME-ORDER
 :region THEME
 :metafunction TEXTUAL)

(gate
 :name end-ORDER
 :inputs (or end-accompaniment
	     end-manner
	     end-matter
	     end-temporal-extent
	     end-spatial-extent
	     end-role
	     end-spatial-location
	     end-temporal-location)
 :outputs ((1.0 end-ORDER
		(ORDERatend end)))
 :selector CHOICE-MASTER
 :old-feature THEME-ORDER
 :region THEME
 :metafunction TEXTUAL)






(gate
 :name initial-ORDER-standard
 :inputs (and declarative
	      (or initial-accompaniment
		  initial-manner
		  initial-matter
		  initial-temporal-extent
		  initial-spatial-extent
		  initial-role
		  iniital-spatial-location
		  initial-temporal-location))
 :outputs ((1.0 initial-ORDER-standard
		(order-anchor-second initial mood)))
 :selector CHOICE-MASTER
 :old-feature THEME-ORDER
 :region THEME
 :metafunction TEXTUAL)

(gate
 :name initial-ORDER-finsubj
 :inputs (and (or finite-subject
		  finite-negator-subject
		  finite-subject-negator)
	      (or initial-accompaniment
		  initial-manner
		  initial-matter
		  initial-temporal-extent
		  initial-spatial-extent
		  initial-role
		  iniital-spatial-location
		  initial-temporal-location))
 :outputs ((1.0 initial-ORDER-finsubj
		(order-anchor-second initial finite)))
 :selector CHOICE-MASTER
 :old-feature THEME-ORDER
 :region THEME
 :metafunction TEXTUAL)

(gate
 :name initial-ORDER-subintr
 :inputs (and explicit-subject-elaborating
	      (or initial-accompaniment
		  initial-manner
		  initial-matter
		  initial-temporal-extent
		  initial-spatial-extent
		  initial-role
		  iniital-spatial-location
		  initial-temporal-location))
 :outputs ((1.0 initial-ORDER-subintr
		(order-anchor-second initial subjectintroducer)))
 :selector CHOICE-MASTER
 :old-feature THEME-ORDER
 :region THEME
 :metafunction TEXTUAL)

